#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "lexer.h"

static char operators[] = "+-*/()";

struct lexer *lexer_init(FILE *input)
{
    struct lexer *lex;

    /* Allocate lexer struct, return 0 on failure */
    if (!(lex = malloc(sizeof(struct lexer))))
	return 0;

    lex->input = input;
    lex->offset = 0;

    return lex;
}

void lexer_destroy_lexeme(struct lexeme *lexeme)
{
    if (lexeme)
	free(lexeme);
}

void lexer_destroy(struct lexer *lex)
{
    if (lex)
	free(lex);
}


/* Parse the incoming number to a double */
static int lexer_operand(struct lexer *lex, struct lexeme *lexeme, int ic)
{
    int decimals = 0;
    int i, c;
    
    lexeme->value = ic - '0';
    
    c = fgetc(lex->input);
    lex->offset++;
    while (c != EOF && (isdigit(c) || c == '.')) {
	if (c == '.' || decimals)
	    decimals++;
	lexeme->value *= 10;
	lexeme->value += c - '0';
	c = fgetc(lex->input);
	lex->offset++;
    }
    ungetc(c, lex->input);

    for (i = 0; i < decimals; i++)
	lexeme->value /= 10;

    return 0;
}

/* check, and parse the operator */
static int lexer_operator(struct lexer *lex, struct lexeme *lexeme, int ic)
{
    int i;

    for (i = 0; i < strlen(operators); i++)
	if (ic == operators[i]) {
	    lexeme->operator = ic;
	    lexeme->opernum = i;
	}

    return 0;
}

struct lexeme *lexer_next(struct lexer *lex)
{
    struct lexeme *lexeme;
    int c, stat;

    /* Allocate lexeme structure */
    if (!(lexeme = malloc(sizeof(struct lexeme))))
	return 0;

    do {
	c = fgetc(lex->input);
	if (c == EOF || c == '\n') {
	    free(lexeme);
	    return 0;
	}
	lex->offset++;
    } while (isspace(c));
    lexeme->offset = lex->offset - 1;

    if (isdigit(c)) {
	lexeme->type = LEX_OPERAND;
	stat = lexer_operand(lex, lexeme, c);
    } else {
	lexeme->type = LEX_OPERATOR;
	stat = lexer_operator(lex, lexeme, c);
    }

    if (stat) {
	free(lexeme);
	return 0;
    } else
	return lexeme;
}
